import 'dart:async';
import 'dart:io';
import 'dart:js';
import 'package:dart_jsonwebtoken/dart_jsonwebtoken.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:grpc/src/client/transport/http2_credentials.dart'
    show ChannelCredentials;
import 'package:jwt_decoder/jwt_decoder.dart';
import 'package:nuntio_core/credentials/credentials.dart';
import 'package:nuntio_core/models/shared.pb.dart';
import 'package:nuntio_hera/hera/hera_extension.dart';
import 'package:nuntio_core/models/hera.pbgrpc.dart';
import 'package:nuntio_core/models/nuntio.pb.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:url_strategy/url_strategy.dart';

import 'package:uni_links/uni_links.dart';
import 'package:synchronized/synchronized.dart';

import 'package:nuntio_hera/hera/channel/hera_channel.dart'
    if (dart.library.html) 'package:nuntio_hera/hera/channel/hera_channel_web.dart';

const String _accessTokenKey = "hera-access-token";
const String _refreshTokenKey = "hera-refresh-token";

class Hera {
  // Hera
  static late HeraClient _heraClient;
  static String _heraPublicKey = "";
  static HeraConfig? _currentConfig;
  static late final String _apiUrl;
  static late final String _namespace;

  static final Lock _lock = Lock();

  static late Function onLogout;
  static late Function onAuthenticated;

  static late TransportCredentials _transportCredentials;

  static late List<OidcProviderType> oidcProviders;
  static late HeraConfig config;

  static const FlutterSecureStorage _secureStorage = FlutterSecureStorage(
    aOptions: AndroidOptions(
      encryptedSharedPreferences: true,
    ),
    webOptions: WebOptions(),
  );

  // Current user
  static User _currentUser = User();
  static String _accessToken = "";
  static String _refreshToken = "";

  // Analytics
  static StreamSubscription? _sub;
  static late String _redirectUrl;

  static Future<void> initialize({
    required String namespace,
    required String apiUrl,
    TransportCredentials? credentials,
    bool? debug,
    String? redirectUrl,
    List<OidcProviderType>? excludeOidcProviders,
    List<LoginType>? excludeAuthMechanisms,
  }) async {
    setPathUrlStrategy();
    _apiUrl = apiUrl;
    _namespace = namespace;

    _redirectUrl = redirectUrl ?? (kIsWeb ? "${Uri.base}/callback" : "");
    _transportCredentials =
        credentials ?? CloudCredentials(apiUrl: Uri.parse(_apiUrl));
    _heraClient =
        await HeraChannel.publicServiceClient(_apiUrl, _transportCredentials);
    // get public key used to validate tokens client side
    HeraRequest publicKeysReq = HeraRequest(namespace: _namespace);
    HeraResponse heraResp = await _heraClient.getPublicKey(publicKeysReq);
    _heraPublicKey = heraResp.publicKey;
    // get config
    config = await initializeAuth();
    // init oidc providers
    config.oidcProviders.removeWhere((key, value) =>
        !value.enabled ||
        excludeOidcProviders?.contains(value.type) == true ||
        excludeAuthMechanisms?.contains(LoginType.OIDC_LOGIN) == true);
    // init login methods
    config.supportedLoginMechanisms.removeWhere(
        (element) => excludeAuthMechanisms?.contains(element) == true);
  }

  static setOnAuthenticated(Function function) {
    onAuthenticated = function;
  }

  static setOnLogout(Function function) {
    onLogout = function;
  }

  static User currentUser() {
    if (_currentUser.id.isNotEmpty) {
      return _currentUser;
    }
    return User();
  }

  static void _setCurrentUser(User currentUser) async {
    _currentUser = currentUser;
  }

  static get redirectUrl => _redirectUrl;

  static Future<String> getAccessToken() async {
    String accessToken = _accessToken;
    if (_accessToken.isEmpty) {
      accessToken = await _secureStorage.read(key: _accessTokenKey) ?? "";
      if (accessToken.isEmpty) return "";
      _accessToken = accessToken;
    }
    try {
      if (JwtDecoder.getRemainingTime(accessToken).inMinutes > 1) {
        try {
          if (_heraPublicKey == "") {
            throw Exception("missing required Hera public key");
          }
          JWT.verify(accessToken, RSAPublicKey(_heraPublicKey));
          return accessToken; // we have a valid access token with time left
        } catch (e) {
          if (kDebugMode) {
            print("could not verify token with err$e");
          }
          return "";
        }
      } else {
        String refreshToken = await getRefreshToken();
        if (refreshToken.isEmpty) {
          return "";
        }
        // token is expired - refresh
        HeraRequest req = HeraRequest()
          ..namespace = _namespace
          ..authToken = AuthToken(userToken: accessToken);
        Token token = Token()
          ..deviceInfo = (await HeraExtenstion.getDeviceInfo())
          ..refreshToken = _refreshToken;
        req.token = token;
        HeraResponse refreshResp = await _heraClient.refreshToken(req);
        _setAccessToken(refreshResp.token.accessToken);
        _setRefreshToken(refreshResp.token.refreshToken);
        return refreshResp.token.accessToken;
      }
    } catch (e) {
      await _secureStorage.delete(key: _refreshTokenKey);
      if (kDebugMode) {
        print("could not refresh token with err: $e");
      }
    }
    return "";
  }

  static void _setAccessToken(String accessToken) async {
    _accessToken = accessToken;
    await _lock.synchronized(() async {
      await _secureStorage.write(key: _accessTokenKey, value: accessToken);
    });
  }

  static Future<String> getRefreshToken() async {
    if (_refreshToken != "") {
      return _refreshToken;
    }
    try {
      String? refreshToken = await _secureStorage.read(key: _refreshTokenKey);
      if (refreshToken != null && refreshToken != "") {
        _refreshToken = refreshToken;
        return refreshToken;
      }
    } catch (e) {
      if (kDebugMode) {
        print(e.toString());
      }
    }
    return "";
  }

  static void _setRefreshToken(String refreshToken) async {
    _refreshToken = refreshToken;
    await _lock.synchronized(() async {
      await _secureStorage.write(key: _refreshTokenKey, value: refreshToken);
    });
  }

  static Future<String> uploadProfileImage({
    required Uint8List data,
  }) async {
    HeraRequest req = HeraRequest()
      ..namespace = _namespace
      ..authToken = AuthToken(userToken: await getAccessToken())
      ..user = (currentUser());
    HeraResponse res = await _heraClient.uploadProfileImage(
      HeraExtenstion.imageStream(data, req, (_) => HeraRequest(data: _)),
    );
    _currentUser.profile.profileImage = res.user.profile.profileImage;
    return res.user.profile.profileImage.url;
  }

  static Future<User> createUser({
    UserProfile? profile,
    UserIdentifier? identifier,
    String? password,
  }) async {
    HeraRequest req = HeraRequest()..namespace = _namespace;
    User user = userWithIdentifiers(identifier);
    user.profile = profile ?? UserProfile();
    user.password = Hash(body: password ?? "");
    req.user = user;
    try {
      HeraResponse resp = await _heraClient.create(req);
      return resp.user;
    } catch (e) {
      if (kDebugMode) {
        print("could not create user with err: $e");
      }
      rethrow;
    }
  }

  static Future<LoginSession> login({
    required UserIdentifier identifier,
    required String password,
  }) async {
    HeraRequest req = HeraRequest()..namespace = _namespace;
    User user = userWithIdentifiers(identifier);
    req.user = user;
    user.password = Hash(body: password);
    // analytical information about token
    Token token = Token();
    token.deviceInfo = await HeraExtenstion.getDeviceInfo();
    req.token = token;
    try {
      HeraResponse resp = await _heraClient.login(req);
      if (resp.loginSession.loginStatus != LoginStatus.AUTHENTICATED) {
        return resp.loginSession;
      }
      if (resp.token.accessToken == "" || resp.token.refreshToken == "") {
        throw Exception("token is null. Contact info@nuntio.io");
      }
      if (resp.user.id == "") {
        throw Exception("user is null. Contact info@nuntio.io");
      }
      _setCurrentUser(resp.user);
      _setAccessToken(resp.token.accessToken);
      _setRefreshToken(resp.token.refreshToken);
    } catch (e) {
      if (kDebugMode) {
        print("could not login user: $e");
      }
      rethrow;
    }
    onAuthenticated();
    return LoginSession()..loginStatus = LoginStatus.AUTHENTICATED;
  }

  static Future<LoginSession> loginWithOidc({
    required OidcProviderType type,
    required String authorizationUrl,
  }) async {
    // validate redirect url
    if (_redirectUrl.isEmpty) {
      throw Exception("redirect url is empty");
    } else if ((kIsWeb && Uri.tryParse(_redirectUrl) == null)) {
      throw Exception("redirect url is invalid");
    }
    await HeraExtenstion.redirect(Uri.parse(authorizationUrl));
    if (kIsWeb) {
      return LoginSession()..loginStatus = LoginStatus.NOT_AUTHENTICATED;
    }
    final Completer<Uri> completer = Completer<Uri>();
    _sub = linkStream.listen((String? uri) async {
      if (uri.toString().startsWith(_redirectUrl)) {
        if (Platform.isIOS || Platform.isAndroid || Platform.isFuchsia) {
          await closeInAppWebView();
        }
        completer.complete(Uri.parse(uri!));
      }
    });
    Uri responseUrl = await completer.future;
    _sub?.cancel();
    HeraRequest req = HeraRequest()..namespace = _namespace;
    req.oidcProvider = OidcProvider()..type = type;
    String? accessToken = responseUrl.queryParameters["access_token"];
    String? refreshToken = responseUrl.queryParameters["access_token"];
    String? userId = responseUrl.queryParameters["user_id"];
    if ((accessToken ?? "") == "" || (refreshToken ?? "") == "") {
      throw Exception(
          "user id, access or refresh token is null. Contact info@nuntio.io");
    }
    if (userId == "") {
      throw Exception("user is null. Contact info@nuntio.io");
    }
    var response = await _heraClient
        .get(HeraRequest(authToken: AuthToken(userToken: accessToken)));
    _setCurrentUser(response.user);
    _setAccessToken(accessToken!);
    _setRefreshToken(refreshToken!);
    onAuthenticated();
    return LoginSession()..loginStatus = LoginStatus.AUTHENTICATED;
  }

  static Future setTokenPair({
    String? accessToken,
    String? refreshToken,
  }) async {
    if (accessToken?.isEmpty ?? true) {
      throw Exception("access token is empty");
    }
    if (refreshToken?.isEmpty ?? true) {
      throw Exception("refresh token is empty");
    }
    var response = await _heraClient
        .get(HeraRequest(authToken: AuthToken(userToken: accessToken)));
    _setCurrentUser(response.user);
    _setAccessToken(accessToken ?? "");
    _setRefreshToken(refreshToken ?? "");
    onAuthenticated();
  }

  static Future<void> logout() async {
    if (await isAuthenticated() == false) {
      throw Exception("user is currently not logged in");
    }
    try {
      String accessToken = await getAccessToken();
      _secureStorage.delete(key: _accessTokenKey);
      _secureStorage.delete(key: _refreshTokenKey);
      _currentUser = User();
      _accessToken = "";
      _refreshToken = "";
      HeraRequest req = HeraRequest()
        ..namespace = _namespace
        ..authToken = AuthToken(userToken: accessToken)
        ..token = (Token()..accessToken = accessToken);
      await _heraClient.logout(req);
      onLogout();
    } catch (e) {
      if (kDebugMode) {
        print("could not logout user: $e");
      }
      rethrow;
    }
  }

  static Future<void> verifyEmail({
    UserIdentifier? identifier,
    required String code,
  }) async {
    User user = userWithIdentifiers(identifier);
    if (code == "") {
      throw Exception("email verification code is empty");
    }
    HeraRequest req = HeraRequest()
      ..namespace = _namespace
      ..verificationCode = (VerificationCode()..code = Hash(body: code));
    req.user = user;
    await _heraClient.verifyEmail(req);
  }

  static Future<VerificationCode> sendResetPasswordEmail({
    required String email,
  }) async {
    User user = User()..contactInfo = UserContactInfo(email: email);
    HeraRequest req = HeraRequest()..namespace = _namespace;
    req.user = user;
    return (await _heraClient.sendResetPasswordEmail(req)).verificationCode;
  }

  static Future<VerificationCode> sendResetPasswordText({
    required String phoneNumber,
  }) async {
    User user = User()..contactInfo = UserContactInfo(phone: phoneNumber);
    HeraRequest req = HeraRequest()..namespace = _namespace;
    req.user = user;
    return (await _heraClient.sendResetPasswordText(req)).verificationCode;
  }

  static Future<void> resetPassword({
    required String userId,
    required String code,
    required String newPassword,
  }) async {
    User user = User()..id = userId;
    HeraRequest req = HeraRequest()
      ..namespace = _namespace
      ..verificationCode = (VerificationCode(code: Hash(body: code)))
      ..update = (User()..password = Hash(body: newPassword));
    req.user = user;
    await _heraClient.resetPassword(req);
  }

  static Future<void> verifyPhone({
    UserIdentifier? identifier,
    required String code,
  }) async {
    User user = userWithIdentifiers(identifier);
    if (code == "") {
      throw Exception("email verification code is empty");
    }
    HeraRequest req = HeraRequest()..namespace = _namespace;
    VerificationCode verificationCode = VerificationCode();
    verificationCode.code = Hash(body: code);
    req.verificationCode = verificationCode;
    req.user = user;
    req.namespace = _namespace;
    await _heraClient.verifyPhone(req);
  }

  static Future<bool> isAuthenticated() async {
    String accessToken = await getAccessToken();
    bool isAuthenticated = accessToken.isNotEmpty;
    if (kDebugMode) {
      print(isAuthenticated);
    }
    if (isAuthenticated) {
      try {
        HeraResponse response = await _heraClient.validateToken(HeraRequest(
            token: Token()..accessToken = accessToken,
            authToken: AuthToken(userToken: accessToken)));
        response = await _heraClient.get(HeraRequest(
          authToken: AuthToken(userToken: accessToken),
        ));
        _setCurrentUser(response.user);
      } catch (e) {
        if (kDebugMode) {
          print(e.toString());
        }
        isAuthenticated = false;
        _setAccessToken("");
        _setRefreshToken("");
      }
    }
    return isAuthenticated;
  }

  static Future<HeraConfig> initializeAuth() async {
    currentUser();
    HeraRequest req = HeraRequest()
      ..namespace = _namespace
      ..redirectAddress = _redirectUrl;
    try {
      _currentConfig ??= (await _heraClient.initializeAuth(req)).config;
      return _currentConfig!;
    } catch (e) {
      if (kDebugMode) {
        print(e);
      }
      rethrow;
    }
  }
}

class NoTransportCredentials implements TransportCredentials {
  @override
  Future<ChannelCredentials> getTransportCredentials() async {
    return const ChannelCredentials.insecure();
  }
}

User userWithIdentifiers(UserIdentifier? identifier) => User(
      id: identifier?.id,
      username: identifier?.username,
      contactInfo: UserContactInfo(
        email: identifier?.email,
        phone: identifier?.phone,
      ),
    );
