import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:nuntio_hera/hera_app/debug.dart';

/// Defines the localized resource values used by the Hera widgets.
///
/// See also:
///
///  * [DefaultHeraLocalizations], the default, English-only, implementation
///    of this interface.
///  * [GlobalHeraLocalizations], which provides Hera localizations for
///    many languages.
abstract class HeraLocalizations {
  /// The login title.
  String get loginTitle;

  /// The login sub title.
  String get loginSubtitle;

  /// The register title.
  String get registerTitle;

  /// The register sub title.
  String get chooseSignupMethodTitle;

  /// The "or login with" text between oidc and login
  String get orLoginText;

  /// The label above the identifier text field
  String get identifierLabel;

  String get resetPasswordIdentifierLabel;

  String get resetPasswordIdentifierHint;

  String get sendResetPasswordTitle;

  String get sendResetPasswordSubtitle;

  /// The hint in the identifier text field
  String get identifierHint;

  /// The label above the password text field
  String get passwordLabel;

  /// The hint in the password text field
  String get passwordHint;

  /// The label above the repeat password text field
  String get repeatPasswordLabel;

  /// The hint in the repeat password text field
  String get repeatPasswordHint;

  /// The label above the email text field
  String get emailLabel;

  /// The hint in the email text field
  String get emailHint;

  /// The label above the phone text field
  String get phoneLabel;

  /// The hint in the phone text field
  String get phoneHint;

  /// The label above the username text field
  String get usernameLabel;

  /// The hint in the username text field
  String get usernameHint;

  /// The text inside the forgot password button
  String get forgotPasswordButton;

  /// The text inside the login button
  String get loginButton;

  /// The text inside the register button
  String get registerButton;

  /// The text inside the don't have an account button
  String get noAccountButton;

  /// The text inside the register by email button
  String get registerByEmailButton;

  /// The text inside the register by phone button
  String get registerByPhoneButton;

  /// The text inside the register by phone button
  String get registerByUsernameButton;

  /// The text inside the register by phone button
  String get registerSubtitle;

  /// The text inside the register by phone button
  String get alreadyHaveAnAccount;

  String get success;

  String get failure;

  String get invalidEmailError;

  String get invalidPhoneError;

  String get invalidUsernameError;

  String get passwordMustContainNumberError;

  String get passwordMustContainSpecialCharError;

  String get passwordMustMin8LengthError;

  String get passwordDoNotMatchError;

  String get passwordMustContainUpperError;

  String get invalidIdentifierError;

  String get invalidPasswordError;

  String get verifyEmailTitle;

  String get verifyEmailSubtitle;

  String get verifyPhoneTitle;

  String get verifyPhoneSubtitle;

  String get verifyButton;

  String get backToLogin;

  String get next;

  String get resetPasswordTitle;

  String get resetPasswordSubtitle;

  /// The `HeraLocalizations` from the closest [Localizations] instance
  /// that encloses the given context.
  ///
  /// If no [HeraLocalizations] are available in the given `context`, this
  /// method throws an exception.
  ///
  /// This method is just a convenient shorthand for:
  /// `Localizations.of<HeraLocalizations>(context, HeraLocalizations)!`.
  ///
  /// References to the localized resources defined by this class are typically
  /// written in terms of this method. For example:
  ///
  /// ```dart
  /// label: HeraLocalizations.of(context).editMessageButtonLabel,
  /// ```
  static HeraLocalizations of(BuildContext context) {
    assert(debugCheckHasHeraLocalizations(context));
    return Localizations.of<HeraLocalizations>(context, HeraLocalizations)!;
  }
}

class _HeraLocalizationsDelegate
    extends LocalizationsDelegate<HeraLocalizations> {
  const _HeraLocalizationsDelegate();

  @override
  bool isSupported(Locale locale) => locale.languageCode == 'en';

  @override
  Future<HeraLocalizations> load(Locale locale) =>
      DefaultHeraLocalizations.load(locale);

  @override
  bool shouldReload(_HeraLocalizationsDelegate old) => false;

  @override
  String toString() => 'DefaultHeraLocalizations.delegate(en_US)';
}

/// US English strings for the Hera widgets.
///
/// See also:
///
///  * [GlobalHeraLocalizations], which provides Hera localizations for
///    many languages.
class DefaultHeraLocalizations implements HeraLocalizations {
  /// Constructs an object that defines the Hera widgets' localized strings
  /// for US English (only).
  ///
  /// [LocalizationsDelegate] implementations typically call the static [load]
  /// function, rather than constructing this class directly.
  const DefaultHeraLocalizations();

  @override
  String get loginTitle => 'Login';

  @override
  String get loginSubtitle => 'Fill in the details to login';

  @override
  String get forgotPasswordButton => "Forgot your password?";

  @override
  String get identifierHint => "Enter your identifier";

  @override
  String get identifierLabel => "Identifier";

  @override
  String get loginButton => "Login";

  @override
  String get noAccountButton => "Don't have an account?";

  @override
  String get orLoginText => "or login with";

  @override
  String get passwordHint => "Enter your password";

  @override
  String get passwordLabel => "Password";

  @override
  String get chooseSignupMethodTitle => "Chose a sign up method to register";

  @override
  String get registerTitle => "Register";

  @override
  String get registerByEmailButton => "Register by email";

  @override
  String get registerByPhoneButton => "Register by phone";

  @override
  String get registerByUsernameButton => "Register by username";

  @override
  String get emailHint => "Enter your email";

  @override
  String get emailLabel => "Email";

  @override
  String get phoneHint => "Enter your phone number";

  @override
  String get phoneLabel => "Phone";

  @override
  String get usernameHint => "Enter your username";

  @override
  String get usernameLabel => "Username";

  @override
  String get repeatPasswordHint => "Enter your password again";

  @override
  String get repeatPasswordLabel => "Repeat password";

  @override
  String get registerButton => "Register";

  @override
  String get registerSubtitle =>
      "Fill our the details below to create an account";

  @override
  String get alreadyHaveAnAccount => "Already have an account?";

  @override
  String get failure => "Failure";

  @override
  String get success => "Success";

  @override
  String get invalidEmailError => "Please enter a valid email";

  @override
  String get invalidPhoneError => "Please enter a valid phone number";

  @override
  String get invalidUsernameError => "Please enter a valid username";

  @override
  String get passwordDoNotMatchError => "Passwords don't match";

  @override
  String get passwordMustContainNumberError => "Password must contain a number";

  @override
  String get passwordMustContainSpecialCharError =>
      "Password must contain a special char";

  @override
  String get passwordMustContainUpperError =>
      "Password must contain an uppercase letter";

  @override
  String get passwordMustMin8LengthError => "Min. password length is 8";

  @override
  String get invalidIdentifierError => "Invalid identifier";

  @override
  String get invalidPasswordError => "Invalid password";

  @override
  String get verifyEmailTitle => "Verify your email";

  @override
  String get verifyEmailSubtitle =>
      "We have sent a verification code to your email. Enter the code below to verify your account.";

  @override
  String get verifyPhoneTitle => "Verify your phone number";

  @override
  String get verifyPhoneSubtitle =>
      "We have sent a verification code to your phone. Enter the code below to verify your account.";

  @override
  String get verifyButton => "Verify";

  @override
  String get backToLogin => "Back to login";

  @override
  String get resetPasswordIdentifierHint => "Email or phone";

  @override
  String get resetPasswordIdentifierLabel => "Enter your email or phone";

  @override
  String get sendResetPasswordTitle => "Reset password";

  @override
  String get sendResetPasswordSubtitle =>
      "Enter your accounts email or phone number to receive a reset password code.";

  @override
  String get next => "Next";

  @override
  String get resetPasswordSubtitle =>
      "We have sent you a reset password code. Enter it below together with your new password to reset it.";

  @override
  String get resetPasswordTitle => "Reset password";

  /// A [LocalizationsDelegate] that uses [DefaultHeraLocalizations.load]
  /// to create an instance of this class.
  static const LocalizationsDelegate<HeraLocalizations> delegate =
      _HeraLocalizationsDelegate();

  /// Creates an object that provides US English resource values for the Hera
  /// library widgets.
  ///
  /// The [locale] parameter is ignored.
  ///
  /// This method is typically used to create a [LocalizationsDelegate].
  static Future<HeraLocalizations> load(Locale locale) {
    return SynchronousFuture<HeraLocalizations>(
        const DefaultHeraLocalizations());
  }
}
