import 'dart:async';
import 'package:flutter/material.dart';
import 'package:nuntio_hera/hera/hera.dart';
import 'package:pinput/pinput.dart';
import 'package:nuntio_core/models/hera.pb.dart';
import 'package:nuntio_core/components/iconed_button.dart';
import 'package:nuntio_core/components/nuntio_state_button.dart';
import 'package:nuntio_core/components/nuntio_text_field.dart';
import 'package:nuntio_core/components/whatever_image.dart';
import 'hera_icons.dart';
import 'hera_localizations.dart';

class ResetPasswordScreenArguments {
  final String userId;
  final DateTime expiresAt;
  ResetPasswordScreenArguments(this.userId, this.expiresAt);
}

class ResetPasswordScreen extends StatefulWidget {
  static const id = "reset-password";
  const ResetPasswordScreen(this.arguments, {super.key});

  final ResetPasswordScreenArguments arguments;

  @override
  State<ResetPasswordScreen> createState() => _ResetPasswordScreenState();
}

class _ResetPasswordScreenState extends State<ResetPasswordScreen> {
  final _verifyCodeController = TextEditingController();
  DateTime _now = DateTime.now();
  late Timer _timer;

  final _formKey = GlobalKey<FormState>();

  final TextEditingController _passwordController = TextEditingController();
  final TextEditingController _repeatController = TextEditingController();

  ButtonState _buttonState = ButtonState.idle;

  _ResetPasswordScreenState() {
    _timer = Timer.periodic(Duration(seconds: 1), (Timer t) {
      setState(() {
        _now = DateTime.now();
      });
      if (widget.arguments.expiresAt.difference(_now).inSeconds <= 0) {
        Navigator.of(context).pop();
      }
    });
  }

  @override
  void dispose() {
    _timer.cancel();
    super.dispose();
  }

  String formatTime(int totalSeconds) {
    final duration = Duration(seconds: totalSeconds);
    final minutes = duration.inMinutes;
    final seconds = totalSeconds % 60;

    final minutesString = '$minutes'.padLeft(2, '0');
    final secondsString = '$seconds'.padLeft(2, '0');
    return '$minutesString:$secondsString';
  }

  Future<void> resetPassword() async {
    if (_formKey.currentState?.validate() == true) {
      setState(() {
        _buttonState = ButtonState.loading;
      });
      Hera.resetPassword(
        userId: widget.arguments.userId,
        code: _verifyCodeController.text,
        newPassword: _passwordController.text,
      ).then((value) {
        setState(() {
          _buttonState = ButtonState.success;
        });
        Timer(Duration(seconds: 2), () {
          Navigator.of(context).pop();
        });
      }).catchError((err) {
        setState(() {
          _buttonState = ButtonState.fail;
        });
        Timer(Duration(seconds: 2), () {
          setState(() {
            _buttonState = ButtonState.idle;
          });
        });
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    var size = MediaQuery.of(context).size;
    return Scaffold(
      body: SingleChildScrollView(
        child: Container(
          constraints: BoxConstraints(minHeight: size.height, maxWidth: 500),
          child: SafeArea(
            child: Center(
              child: Container(
                constraints: BoxConstraints(maxWidth: 450),
                margin: const EdgeInsets.only(
                    left: 20, right: 20, bottom: 20, top: 0),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.center,
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    WhatEverImage(Hera.config.logo.url, 100),
                    SizedBox(
                      height: 20,
                    ),
                    Text(
                      HeraLocalizations.of(context).resetPasswordTitle,
                      style: Theme.of(context).textTheme.displayMedium,
                    ),
                    SizedBox(height: 10),
                    Text(
                      HeraLocalizations.of(context).resetPasswordSubtitle,
                      style: Theme.of(context).textTheme.headlineSmall,
                      textAlign: TextAlign.center,
                    ),
                    SizedBox(
                      height: 20,
                    ),
                    Text(
                      formatTime(widget.arguments.expiresAt.difference(_now).inSeconds),
                      style: Theme.of(context).textTheme.headlineSmall,
                    ),
                    SizedBox(
                      height: 20,
                    ),
                    Card(
                      elevation: 0,
                      child: Pinput(
                        length: 6,
                        obscureText: false,
                        enabled: true,
                        hapticFeedbackType: HapticFeedbackType.mediumImpact,
                        // set this based on expiration
                        keyboardType: TextInputType.number,
                        animationDuration: const Duration(milliseconds: 300),
                        controller: _verifyCodeController,
                        textInputAction: TextInputAction.done,
                        androidSmsAutofillMethod:
                            AndroidSmsAutofillMethod.smsRetrieverApi,
                      ),
                    ),
                    SizedBox(height: 20),
                    Form(
                      key: _formKey,
                      child: Column(
                        children: [
                          NuntioTextField(
                            prefix: HeraIcons.instance.password,
                            label: HeraLocalizations.of(context).passwordLabel,
                            hint: HeraLocalizations.of(context).passwordHint,
                            controller: _passwordController,
                            textInputType: TextInputType.text,
                            textInputAction: TextInputAction.next,
                            obscureText: true,
                            validator: (text) {
                              if (!Hera.config.validatePassword) return null;
                              if (text == null ||
                                  text.isEmpty ||
                                  !_passwordController.text
                                      .contains(RegExp(r'[0-9]'))) {
                                return HeraLocalizations.of(context)
                                    .passwordMustContainNumberError;
                              } else if (!_passwordController.text.contains(
                                  RegExp(r'[!@#$%^&*(),.?":{}|<>]'))) {
                                return HeraLocalizations.of(context)
                                    .passwordMustContainSpecialCharError;
                              } else if (_passwordController.text.length < 8) {
                                return HeraLocalizations.of(context)
                                    .passwordMustMin8LengthError;
                              }
                              return null;
                            },
                          ),
                          SizedBox(height: 10),
                          NuntioTextField(
                            prefix: HeraIcons.instance.repeatPassword,
                            label: HeraLocalizations.of(context)
                                .repeatPasswordLabel,
                            hint: HeraLocalizations.of(context)
                                .repeatPasswordHint,
                            controller: _repeatController,
                            textInputType: TextInputType.text,
                            textInputAction: TextInputAction.done,
                            obscureText: true,
                            validator: (_) {
                              if (_passwordController.text !=
                                  _repeatController.text) {
                                return HeraLocalizations.of(context)
                                    .passwordDoNotMatchError;
                              }
                              return null;
                            },
                          ),
                        ],
                      ),
                    ),
                    SizedBox(height: 20),
                    SizedBox(
                      width: double.infinity,
                      height: Theme.of(context).buttonTheme.height,
                      child: NuntioStateButton.icon(
                        context,
                        iconedButtons: {
                          ButtonState.idle: IconedButton(
                            text: HeraLocalizations.of(context).verifyButton,
                            icon: Icon(
                              HeraIcons.instance.login,
                            ),
                            color: Theme.of(context).primaryColor,
                          ),
                          ButtonState.loading: IconedButton(
                            text: "",
                            color: Theme.of(context).primaryColor,
                          ),
                          ButtonState.fail: IconedButton(
                            text: HeraLocalizations.of(context).failure,
                            icon: Icon(Icons.cancel),
                            color: Colors.red.shade300,
                          ),
                          ButtonState.success: IconedButton(
                            text: HeraLocalizations.of(context).success,
                            icon: Icon(Icons.check_circle),
                            color: Colors.green.shade400,
                          )
                        },
                        state: _buttonState,
                        onPressed: () => resetPassword(),
                      ),
                    ),
                    Padding(
                      padding: const EdgeInsets.only(top: 10),
                      child: TextButton(
                        onPressed: () => Navigator.of(context).pop(),
                        child: Text(
                          HeraLocalizations.of(context).backToLogin,
                          style: TextStyle(
                            color: Theme.of(context).primaryColor,
                          ),
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
