import 'dart:async';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nuntio_core/models/nuntio.pb.dart';
import 'package:pinput/pinput.dart';
import 'package:nuntio_core/models/hera.pb.dart';
import 'package:nuntio_hera/hera/hera.dart';
import 'package:nuntio_core/components/iconed_button.dart';
import 'package:nuntio_core/components/nuntio_state_button.dart';
import 'package:nuntio_core/components/whatever_image.dart';
import 'hera_icons.dart';
import 'hera_localizations.dart';

class VerificationScreenArguments {
  final String? email;
  final String? phone;
  final String password;
  final DateTime expiresAt;
  final LoginType loginType;

  VerificationScreenArguments(
      this.email, this.phone, this.password, this.expiresAt, this.loginType);
}

class VerificationScreen extends StatefulWidget {
  static const id = "verify";
  const VerificationScreen(
    this.arguments, {
    Key? key,
  }) : super(key: key);

  final VerificationScreenArguments arguments;

  @override
  State<VerificationScreen> createState() => _VerificationScreenState();
}

class _VerificationScreenState extends State<VerificationScreen> {
  final _verifyCodeController = TextEditingController();
  DateTime _now = DateTime.now();
  late Timer _timer;

  ButtonState _buttonState = ButtonState.idle;

  _VerificationScreenState() {
    _timer = Timer.periodic(const Duration(seconds: 1), (Timer t) {
      setState(() {
        _now = DateTime.now();
      });
      if (widget.arguments.expiresAt.difference(_now).inSeconds <= 0) {
        Navigator.of(context).pop();
      }
    });
  }

  @override
  void dispose() {
    _timer.cancel();
    super.dispose();
  }

  String formatTime(int totalSeconds) {
    final duration = Duration(seconds: totalSeconds);
    final minutes = duration.inMinutes;
    final seconds = totalSeconds % 60;

    final minutesString = '$minutes'.padLeft(2, '0');
    final secondsString = '$seconds'.padLeft(2, '0');
    return '$minutesString:$secondsString';
  }

  Future<void> verifyCode() async {
    try {
      setState(() {
        _buttonState = ButtonState.loading;
      });
      if (widget.arguments.loginType == LoginType.EMAIL_PASSWORD) {
        await Hera.verifyEmail(
            code: _verifyCodeController.text,
            identifier: UserIdentifier(email: widget.arguments.email));
      } else if (widget.arguments.loginType == LoginType.PHONE_PASSWORD) {
        await Hera.verifyPhone(
                code: _verifyCodeController.text,
                identifier: UserIdentifier(phone: widget.arguments.phone))
            .then((value) {
          setState(() {
            _buttonState = ButtonState.success;
          });
        });
      }
      setState(() {
        _buttonState = ButtonState.success;
      });
      await Hera.login(
          identifier: UserIdentifier(
            email: widget.arguments.email,
            phone: widget.arguments.phone,
          ),
          password: widget.arguments.password);
      Navigator.of(context).pop();
    } catch (e) {
      setState(() {
        _buttonState = ButtonState.fail;
      });
      Timer(Duration(seconds: 2), () {
        setState(() {
          _buttonState = ButtonState.idle;
        });
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    var size = MediaQuery.of(context).size;
    return Scaffold(
      body: SingleChildScrollView(
        child: Container(
          constraints: BoxConstraints(minHeight: size.height, maxWidth: 500),
          child: SafeArea(
            child: Center(
              child: Container(
                constraints: BoxConstraints(maxWidth: 450),
                margin: const EdgeInsets.only(
                    left: 20, right: 20, bottom: 20, top: 0),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.center,
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    WhatEverImage(Hera.config.logo.url, 100),
                    SizedBox(
                      height: 20,
                    ),
                    Text(
                      widget.arguments.loginType == LoginType.PHONE_PASSWORD
                          ? HeraLocalizations.of(context).verifyPhoneTitle
                          : HeraLocalizations.of(context).verifyEmailTitle,
                      style: Theme.of(context).textTheme.displayMedium,
                    ),
                    SizedBox(height: 10),
                    Text(
                      widget.arguments.loginType == LoginType.PHONE_PASSWORD
                          ? HeraLocalizations.of(context).verifyPhoneSubtitle
                          : HeraLocalizations.of(context).verifyEmailSubtitle,
                      style: Theme.of(context).textTheme.headlineSmall,
                      textAlign: TextAlign.center,
                    ),
                    SizedBox(
                      height: 50,
                    ),
                    Text(
                      formatTime(widget.arguments.expiresAt.difference(_now).inSeconds),
                      style: Theme.of(context).textTheme.headlineSmall,
                    ),
                    SizedBox(
                      height: 50,
                    ),
                    Card(
                      elevation: 0,
                      child: Pinput(
                        length: 6,
                        obscureText: false,
                        enabled: true,
                        hapticFeedbackType: HapticFeedbackType.mediumImpact,
                        // set this based on expiration
                        keyboardType: TextInputType.number,
                        animationDuration: Duration(milliseconds: 300),
                        controller: _verifyCodeController,
                        onSubmitted: (_) => verifyCode(),
                        textInputAction: TextInputAction.done,
                        androidSmsAutofillMethod:
                            AndroidSmsAutofillMethod.smsRetrieverApi,
                      ),
                    ),
                    SizedBox(
                      width: double.infinity,
                      height: Theme.of(context).buttonTheme.height,
                      child: NuntioStateButton.icon(
                        context,
                        iconedButtons: {
                          ButtonState.idle: IconedButton(
                            text: HeraLocalizations.of(context).verifyButton,
                            icon: Icon(
                              HeraIcons.instance.login,
                            ),
                            color: Theme.of(context).primaryColor,
                          ),
                          ButtonState.loading: IconedButton(
                            text: "",
                            color: Theme.of(context).primaryColor,
                          ),
                          ButtonState.fail: IconedButton(
                            text: HeraLocalizations.of(context).failure,
                            icon: Icon(Icons.cancel),
                            color: Colors.red.shade300,
                          ),
                          ButtonState.success: IconedButton(
                            text: HeraLocalizations.of(context).success,
                            icon: Icon(
                              Icons.check_circle,
                            ),
                            color: Colors.green.shade400,
                          )
                        },
                        state: _buttonState,
                        onPressed: () => verifyCode(),
                      ),
                    ),
                    Padding(
                      padding: const EdgeInsets.only(top: 10),
                      child: TextButton(
                        onPressed: () => Navigator.of(context).pop(),
                        child: Text(
                          HeraLocalizations.of(context).backToLogin,
                          style: TextStyle(
                            color: Theme.of(context).primaryColor,
                          ),
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
